package de.mattv.quarry

import mcjty.theoneprobe.api.IProbeHitData
import mcjty.theoneprobe.api.IProbeInfo
import mcjty.theoneprobe.api.IProbeInfoAccessor
import mcjty.theoneprobe.api.ProbeMode
import net.minecraft.core.BlockPos
import net.minecraft.network.chat.TextComponent
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResult
import net.minecraft.world.entity.item.ItemEntity
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.SoundType
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.material.Material
import net.minecraft.world.phys.BlockHitResult

class QuarryBlock : Block(Properties.of(Material.STONE).sound(SoundType.METAL).strength(5.0f, 10000.0f)), EntityBlock, IProbeInfoAccessor {
    override fun newBlockEntity(pos: BlockPos, state: BlockState): BlockEntity? = QuarryMod.QUARRY_TILE.get().create(pos, state)

    override fun <T : BlockEntity?> getTicker(level: Level, state: BlockState, type: BlockEntityType<T>): BlockEntityTicker<T>? =
            if (type == QuarryMod.QUARRY_TILE.get()) QuarryTileTicker() else null

    override fun use(state: BlockState, level: Level, pos: BlockPos, player: Player, hand: InteractionHand, result: BlockHitResult): InteractionResult {
        val be = level.getBlockEntity(pos)
        if (!level.isClientSide() && !player.isSecondaryUseActive && hand == InteractionHand.MAIN_HAND && player.getItemInHand(hand).isEmpty && be is QuarryTile) {
            player.displayClientMessage(TextComponent(be.playerInfo()), true)
            return InteractionResult.PASS
        }
        return super.use(state, level, pos, player, hand, result)
    }

    private fun dropUpgradeItem(level: Level, pos: BlockPos, count: Int, item: Item) {
        val x = pos.x.toDouble()
        val y = pos.y.toDouble()
        val z = pos.z.toDouble()
        var toDrop = count
        while (toDrop > 64) {
            level.addFreshEntity(ItemEntity(level, x, y, z, ItemStack(item, 64)))
            toDrop -= 64
        }
        if (toDrop > 0)
            level.addFreshEntity(ItemEntity(level, x, y, z, ItemStack(item, toDrop)))
    }

    override fun playerDestroy(level: Level, player: Player, pos: BlockPos, state: BlockState, be: BlockEntity?, tool: ItemStack) {
        if (!level.isClientSide() && be is QuarryTile) {
            val toDrop = be.getUpgradesToDrop()
            dropUpgradeItem(level, pos, toDrop.a, QuarryMod.QUARRY_SPEED_UPGRADE.get())
            dropUpgradeItem(level, pos, toDrop.b, QuarryMod.QUARRY_FORTUNE_UPGRADE.get())
        }
        super.playerDestroy(level, player, pos, state, be, tool)
    }

    override fun addProbeInfo(mode: ProbeMode?, info: IProbeInfo?, player: Player?, level: Level?, state: BlockState?, data: IProbeHitData?) {
        val be = level!!.getBlockEntity(data!!.pos)
        if (be is QuarryTile)
            be.probeInfo(info!!)
    }
}